package com.lantone.structure.facade.tran;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.structure.facade.tran.util.CommonAnalysisUtil;
import com.lantone.structure.model.doc.ThreeLevelWardDoc;
import com.lantone.structure.model.label.ThreeLevelWardLabel;
import lombok.extern.slf4j.Slf4j;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description:上级医师查房记录
 * @author: cy
 * @time: 2021/2/20 14:07
 */
@Slf4j
public class ThreeLevelWardTran extends TargetTran {

    @Override
    public Map<String, String> convert(String text) {
        List<ThreeLevelWardDoc> wardRecordDocList = new ArrayList<>();
        ThreeLevelWardDoc wardRecordDoc = new ThreeLevelWardDoc();
        wardRecordDoc.setText(text);
        wardRecordDocList.add(wardRecordDoc);
        inputInfo.setThreeLevelWardDocs(wardRecordDocList);
        Map<String, String> structureMap = cutWord(text);
        wardRecordContrast(inputInfo.getThreeLevelWardDocs(), structureMap);
        wardRecordDoc.setStructureMap(structureMap);
        aiProcess();
        lastSovle(inputInfo.getThreeLevelWardDocs(),structureMap);
        return structureMap;
    }

    private Map<String, String> cutWord(String text) {
        Map<String, String> sourceMap = Maps.newHashMap();
        List<String> titles = CommonAnalysisUtil.sortTitles(
                Lists.newArrayList("鉴别诊断","诊疗计划","诊断考虑","补充诊断","Caprini评分为","注意事项"),
                text
        );
        CommonAnalysisUtil.cutByTitles(text, titles, 0, sourceMap);
        sourceMap.entrySet().removeIf(entry -> entry.getKey().equals("鉴别诊断") || entry.getKey().equals("补充诊断") || entry.getKey().equals("注意事项") || entry.getKey().equals("Caprini评分为") || entry.getKey().equals("诊断考虑"));
        return sourceMap;
    }


    public void wardRecordContrast( List<ThreeLevelWardDoc> wardRecordDoc,Map<String, String> retMap) {
        String text = wardRecordDoc.get(0).getText();
        String fir = text;
        //查房日期时间
        String sumdDate = extractDate(text);
        if(StringUtil.isNotEmpty(sumdDate)){
            retMap.put("查房日期时间",sumdDate);
        }

        String signName= parseString(text);
        if(StringUtil.isNotEmpty(signName) && StringUtil.isNotEmpty(signName.trim()) && signName.length()<15){
            retMap.put("记录人签名",signName);
        }

        if(StringUtil.isNotEmpty(sumdDate)){
            String[] split = text.split(sumdDate);
            fir = text.substring(text.lastIndexOf(split[1]));
            String[] notes = fir.split("查房记录");
            if(notes.length<=1){
                notes = fir.split("日常查房");
            }
            if(notes.length<=1){
                notes = fir.split("查房录");
            }
            fir = fir.substring(fir.lastIndexOf(notes[1]));
        }

        if( StringUtil.isNotEmpty(signName.trim()) && signName.length()<15){
            fir = fir.substring(0,fir.lastIndexOf(signName));
            String flag  = "/";
            if(fir.endsWith("//")){
                flag = "//";
            }
            fir = fir.substring(0,fir.lastIndexOf(flag));
        }

        if(StringUtil.isNotEmpty(fir)){
            retMap.put("查房记录",fir);
        }

        if(StringUtil.isNotEmpty(retMap.get("诊疗计划"))){
            String plan = retMap.get("诊疗计划");
            if(StringUtil.isNotEmpty( retMap.get("记录人签名")) && StringUtil.isNotEmpty( retMap.get("记录人签名").trim())){
                String name = retMap.get("记录人签名").trim();
                if(plan.contains(name)){
                plan = plan.substring(0,plan.lastIndexOf(name));
                }
                if(plan.endsWith("今予")){
                    plan = plan.substring(0,plan.lastIndexOf("今予"));
                }
            }
            if(plan.contains("诊断考虑：")){
                plan = plan.substring(0,plan.lastIndexOf("诊断考虑："));
            }
            if(plan.contains("VTE风险评估")){
                plan = plan.substring(0,plan.lastIndexOf("。")+"。".length());
            }
            String flag  = "/";
            if(fir.endsWith("//")){
                flag = "//";
            }
            if(plan.endsWith(flag)){
                plan = plan.substring(0,plan.lastIndexOf(flag));
            }
            if(plan.endsWith("，")){
                plan = plan.substring(0,plan.lastIndexOf("，"));
            }
               retMap.put("诊疗计划",plan);
           }
        }

    // 取/后
    public static String parseString(String text){
        if(text.contains("/")){
            text = text.substring(text.lastIndexOf("/")+1);
            text= parseString(text);
        }
        return text;
    }

    // 取/前
    public static String parseStr(String text){
        if(text.contains("/")){
            text = text.substring(0,text.lastIndexOf("/"));
            text= parseStr(text);
        }
        return text;
    }

    /**
     * 抽取文本中的第一个时间
     *
     * @param top
     * @return
     */
    public static String extractDate(String top) {
        Pattern pattern = Pattern.compile("[0-9]{4}[-][0-9]{1,2}[-][0-9]{1,2}");
        Matcher matcher = pattern.matcher(top);
        if (matcher.find()) {
            return matcher.group(0);
        } else {
            Pattern p1 = Pattern.compile("[0-9]{4}年[0-9]+月[0-9]+日");
            Matcher m1 = p1.matcher(top);
            if (m1.find()) {
                return m1.group(0);
            }
        }
        return null;
    }

    public static void lastSovle(List<ThreeLevelWardDoc> threeLevelWardDocs, Map<String,String> structureMap) {
        if(ListUtil.isNotEmpty(threeLevelWardDocs) && threeLevelWardDocs.get(0)!=null && threeLevelWardDocs.get(0)
                .getThreeLevelWardLabel().size()>0 && threeLevelWardDocs.get(0).getThreeLevelWardLabel().get(0)!=null){
            ThreeLevelWardLabel threeLevelWardLabel = threeLevelWardDocs.get(0).getThreeLevelWardLabel().get(0);
            String doctorAdviceRecord = threeLevelWardLabel.getDoctorAdviceRecord();
            if(StringUtil.isNotEmpty(doctorAdviceRecord)){
                structureMap.put("医嘱内容",doctorAdviceRecord);
            }
        }
    }
}